@@ -36,6 +36,15 @@ int fatal(char* fmt, ...)
  exit(1);
 }
 
+void downcase(char* ptr)
+{
+ int i;
+ int s;
+ if (!ptr) return;
+ s = strlen(ptr);
+ for (i=0;i<s;i++) if (ptr[i]>='A' && ptr[i]<='Z') ptr[i]+=0x20;
+}
+
 
 int error(char* fmt, ...)
 {
