@@ -14,18 +14,21 @@
 
 #define ANSWER_YES 1
 #define ANSWER_NO  0
-#define MAX_ENTRY_LENGTH  75
-#define MAX_MSG_LENGTH   275
+#define MAX_ENTRY_LENGTH 512
+#define MAX_MSG_LENGTH   512
 
 int gtk_dialog_yes_no(const char*);
 int gtk_dialog_with_2_buttons(const char*, char*, char*);
 int gtk_dialog_yes_no_printf(const char*, ...);
 char* gtk_dialog_gettext(const char*);
 char* gtk_dialog_gettext_printf(const char*, ...);
+char* gtk_dialog_getfile_printf(const char*, ...);
+char* gtk_dialog_getfile(const char*);
 void gtk_dialog_mbox(const char*, const char*);
 void gtk_dialog_printf_with_title(const char*, const char*, ...);
 void gtk_dialog_printf_big_with_title(const char*, const char*, ...);
 void gtk_dialog_printf(const char*, ...);
 void gtk_dialog_printf_big(const char*, ...);
+char* gtk_dialog_getdbs(PGconn*, char*,...);
 
 #endif
