@@ -20,17 +20,20 @@ void disconnect()
  disconnect_db(&connection);
 }
 
+
 static void __dummy( GtkWidget *w,gpointer   data )
 {
   error("Not yet implemented.\n");
 }
 
+
 static void close_results( GtkWidget *w,gpointer   data)
 {
  gtk_main_quit();
  gtk_widget_destroy((GtkWidget*)(data));
 }
 
+
 static void qexec_quit( GtkWidget *w,gpointer   data)
 {
  gtk_main_quit();
@@ -38,13 +41,14 @@ static void qexec_quit( GtkWidget *w,gpointer   data)
  query_exec = NULL;
 }
 
+
 static int menu_quit( GtkWidget *w,gpointer   data)
 {
-  if (gtk_dialog_yes_no("Are you sure?")) 
+  if (gtk_dialog_yes_no("Are you sure?"))
    {
     disconnect();
     gtk_main_quit();
-    if (query_exec) 
+    if (query_exec)
        {
 	qexec_quit(query_exec, NULL);
 	exit(0);
@@ -60,39 +64,132 @@ static void about_box( GtkWidget *w,gpointer   data)
  gtk_dialog_printf("PgXexplorer: version 0.1 alpha", "Written by\nMorgoth DBMA\nPgXexplorer\nCurrently connected to: %s", currentbase);
 }
 
+void fatal_nodb(char*, char*,int);
+	
+static void create_DB_from_file(GtkWidget* w, gpointer data)
+{
+ char *filen,*sfilen,*dbn;
+ char errstr[MAX_QUERY_LENGTH];
+ char dbname[MAXPATH];
+ FILE* f;
+ char*** ptr;
+ int l,row,col;
+ char winname[STDSTRING];
+ dbn = gtk_dialog_gettext_printf("Need DB name to fill from a SQL file:");
+ if (!strcmp(dbn,""))
+    {
+     gtk_dialog_printf("Will not create empty name DB\nTry once again.");
+     return;
+    }
+ strcpy(dbname,dbn);
+ filen = gtk_dialog_getfile("Select SQL dump file (*.SQL)");
+ if (!strcmp(filen,""))
+   {
+    gtk_dialog_printf("Cannot load DB from an empty name\n");
+    return;
+   }
+ l = strlen(filen);
+ sfilen = malloc(l+1);
+ strcpy(sfilen,filen);
+ downcase(sfilen);
+ if (l<5 || sfilen[l-1]!='l' || sfilen[l-2]!='q' || sfilen[l-3] !='s' || sfilen[l-4]!='.') 
+      {
+       gtk_dialog_printf("This is not a .SQL file.\n");
+       free(sfilen);
+       return;
+      }
+ free(sfilen);
+ f = fopen(filen,"r");
+ if (!f)
+   {
+    gtk_dialog_printf("Cannot read from: %s\n", filen);
+    return ;
+   }
+ fclose(f);
+ execute_printf(connection, &ptr, &row, &col, errstr, "CREATE DATABASE %s", dbname);
+ if (row<0 || col<0)
+   {
+    gtk_dialog_printf_big("Error executing query:\nReturn from DBMS: %s\n", errstr);
+    return;
+   }
+ free_p3c(&ptr, row, col);
+ system_printf("psql %s -f %s", dbname, filen);
+ l = gtk_dialog_yes_no_printf("Switch from database\n%s to %s now?", currentbase, dbname);
+ if (l)
+  {
+   disconnect();
+   free(currentbase);
+   sprintf(winname, "PgXexplorer: (no database opened currently)");
+   gtk_window_set_title (GTK_WINDOW(main_wnd), winname);
+   connection = gtk_connect_db(dbname, errstr);
+   if (!connection)
+     {
+      fatal_nodb(dbname,errstr,1);
+      exit(1);
+      return ;
+     }
+   else
+     {
+      currentbase = (char*)malloc(strlen(dbname)+2);
+      strcpy(currentbase, dbname);
+      sprintf(winname, "PgXexplorer: %s", currentbase);
+      gtk_window_set_title (GTK_WINDOW(main_wnd), winname);
+     }
+  }
+}
+
 static void save_database(GtkWidget* w, gpointer data)
 {
  int a;
  char* filename;
+ char* dbname;
+ char dbs[STDSTRING];
  a = gtk_dialog_with_2_buttons("Do You want to save Current database or Other database?", "Current", "Other");
  if (a)
    {
-    filename = gtk_dialog_gettext_printf("Enter filename, %s will be saved to:\n(extension .SQL will be added automatically)", currentbase);
+    /*filename = gtk_dialog_gettext_printf("Enter filename, %s will be saved to:\n(extension .SQL will be added automatically)", currentbase);*/
+    filename = gtk_dialog_getfile_printf("Enter filename, %s will be saved to", currentbase);
     if (!strcmp(filename,""))
       {
 	gtk_dialog_printf("Cannot save %s to empty name.\n", currentbase);
         return;
       }
-    system_printf("pg_dump %s > %s.SQL", currentbase, filename);
+    system_printf("pg_dump -i '%s' > '%s.SQL'", currentbase, filename);
    }
+ else
+ {
+  dbname = gtk_dialog_getdbs(connection, "Which DB save?");
+  strcpy(dbs, dbname);  /* HEADERS WILL USE OLD POINTER IN DIALOGS, SO MUST COPY IT */
+  			/* NOW VARIABLE dbs IS A COPY, AND dbname CAN (NOT MUST) */
+  			/* BE CHANGED BY DIALOGS */
+  filename = gtk_dialog_getfile_printf("Enter filename, %s will be saved to", dbs);
+  if (!strcmp(filename,""))
+      {
+	gtk_dialog_printf("Cannot save %s to empty name.\n", dbs);
+        return;
+      }
+  system_printf("pg_dump -i '%s' > '%s.SQL'", dbs, filename);
+ }
 }
 
 char* get_env(char*);
 
 char *dbenv()
 {
- sprintf(tmpstr,"PGDATA=\"%s\"\nPGPORT=\"%s\"\nPGHOST=\"%s\"", get_env("PGDATA"), get_env("PGPORT"), get_env("PGHOST")); 
+ sprintf(tmpstr,"PGDATA=\"%s\"\nPGPORT=\"%s\"\nPGHOST=\"%s\"", get_env("PGDATA"), get_env("PGPORT"), get_env("PGHOST"));
  return tmpstr;
 }
 
+
 void fatal_nodb(char* dbname, char* errstr, int killed)
 {
- gtk_dialog_printf_big_with_title("PgXexplorer Fatal Error", 
- 	"Cannot connect to DB: %s\nDBMS returned: \n%s\nSystem will now exit.\nENV=%s", 
+ gtk_dialog_printf_big_with_title("PgXexplorer Fatal Error",
+ 	"Cannot connect to DB: %s\nDBMS returned: \n%s\nSystem will now exit.\nENV=\n%s",
   	dbname,errstr,dbenv());
  if(killed) exit(1);
 }
 
+
 void reload_bases(char* db, int ask)
 {
  int a;
@@ -106,7 +203,7 @@ void reload_bases(char* db, int ask)
  if (!a) return;
  disconnect();
  free(currentbase);
- while (!strcmp(dbname,"")) 
+ while (!strcmp(dbname,""))
    {
     gtk_dialog_printf("Cannot switch to an empty DB");
     strcpy(dbname, gtk_dialog_gettext("Null DB name cannot be accepted\nEnter database name: "));
@@ -119,7 +216,7 @@ void reload_bases(char* db, int ask)
     fatal_nodb(dbname,errstr,1);
     return ;
    }
- else 
+ else
    {
     currentbase = (char*)malloc(strlen(dbname)+2);
     strcpy(currentbase, dbname);
@@ -128,6 +225,70 @@ void reload_bases(char* db, int ask)
    }
 }
 
+static void load_database(GtkWidget* w, gpointer data)
+{
+ char* dbname;
+ char winname[STDSTRING];
+ char errstr[MAX_QUERY_LENGTH];
+ dbname = gtk_dialog_getdbs(connection, "Which DB load?");
+ if (!strcmp(dbname,""))
+    {
+     gtk_dialog_printf("Will not load empty DB name");
+     return;
+    }
+ if (!strcmp(dbname,currentbase))
+    {
+     gtk_dialog_printf("This DB: %s, You are currently using", currentbase);
+     return ;
+    }
+ disconnect();
+ free(currentbase);
+ sprintf(winname, "PgXexplorer: (no database opened currently)");
+ gtk_window_set_title (GTK_WINDOW(main_wnd), winname);
+ connection = gtk_connect_db(dbname, errstr);
+ if (!connection)
+     {
+      fatal_nodb(dbname,errstr,1);
+      exit(1);
+      return ;
+     }
+   else
+     {
+      currentbase = (char*)malloc(strlen(dbname)+2);
+      strcpy(currentbase, dbname);
+      sprintf(winname, "PgXexplorer: %s", currentbase);
+      gtk_window_set_title (GTK_WINDOW(main_wnd), winname);
+     }
+}
+
+static void drop_database(GtkWidget* w, gpointer data)
+{
+ char* dbname;
+ char errstr[MAX_QUERY_LENGTH];
+ PGresult *result;
+ dbname = gtk_dialog_getdbs(connection, "Which DB drop?");
+ if (!strcmp(dbname,""))
+    {
+     gtk_dialog_printf("Will not drop empty DB name\nTry once again.");
+     return;
+    }
+ if (!strcmp(dbname,currentbase))
+    {
+     gtk_dialog_printf("Cannot drop DB: %s, currently used!", currentbase);
+     return ;
+    }
+ if (strstr(dbname, "template"))
+   {
+    if (!gtk_dialog_yes_no_printf("Are You sure want to drop\ntemplate DB: %s",dbname)) return ;
+   }
+ execute_printf_query(connection, &result, errstr, "DROP DATABASE %s", dbname);
+ if (!result)
+   {
+    gtk_dialog_printf_big("Error executing query:\nReturn from DBMS: %s\n", errstr);
+    return;
+   }
+}
+
 static void create_database(GtkWidget* w, gpointer data)
 {
  char* dbname;
@@ -162,7 +323,7 @@ static void create_database(GtkWidget* w, gpointer data)
       exit(1);
       return ;
      }
-   else 
+   else
      {
       currentbase = (char*)malloc(strlen(dbname)+2);
       strcpy(currentbase, dbname);
@@ -172,11 +333,13 @@ static void create_database(GtkWidget* w, gpointer data)
   }
 }
 
+
 static void exec_test_prog(GtkWidget* w, gpointer data)
 {
  system("./pgxtest");
 }
 
+
 void gtk_table_out(char* title, char*** str_data, int a, int b)
 {
  GtkWidget* window, *table, *button, *scrolled_window;
@@ -188,7 +351,13 @@ void gtk_table_out(char* title, char*** str_data, int a, int b)
  window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
  gtk_window_set_modal(GTK_WINDOW(window), TRUE);
  gtk_widget_set_usize (GTK_WIDGET(window), 500, 500);
- table = gtk_table_new (b+1, a+1, FALSE); 
+ if (b>16 || a>1024)
+   {
+    gtk_dialog_printf("Result is too big, rows=%d, columns=%d\nwill be truncated!", a, b);
+    b = (b>16 )?16 :b;
+    a = (a>1024)?1024:a;
+   }
+ table = gtk_table_new (b+1, a+1, FALSE);
  for (i=0;i<a;i++)
  for (j=0;j<b;j++)
    {
@@ -218,6 +387,107 @@ void gtk_table_out(char* title, char*** str_data, int a, int b)
  free(entry);
 }
 
+/*                    TEST QUERY EXEC FUNC
+void test_queries()
+{
+ char*** ptr;
+ char errstr[MAX_QUERY_LENGTH];
+ int a,b;
+ ptr=p3c_execute_printf(connection,&a,&b,errstr, "SELECT * FROM %s", "ocena");
+ if (a<0 || b<0)
+ {
+  gtk_dialog_printf("%s", errstr);
+  return ; 
+ }
+ free_p3c(&ptr,a,b);
+}
+*/
+
+void clear_txt_ctrl(GtkWidget* w)
+{
+ int l = gtk_text_get_length(GTK_TEXT(w));
+ gtk_text_set_point(GTK_TEXT(w), 0);
+ gtk_text_forward_delete(GTK_TEXT(w), l);
+}
+
+static void load_proc(GtkWidget* w, gpointer txt)
+{
+ char* filen;
+ FILE* f;
+ int l;
+ char* str;
+ filen = gtk_dialog_getfile("Select query file (*.QUE)");
+ if (!strcmp(filen,""))
+   {
+    gtk_dialog_printf("Cannot load query from an empty name\n");
+    return;
+   }
+ f = fopen(filen,"r");
+ if (!f)
+   {
+    gtk_dialog_printf("Cannot read from: %s\n", filen);
+    return ;
+   }
+ clear_txt_ctrl(txt);
+ fseek(f,0,SEEK_END);
+ l=ftell(f);
+ fseek(f,0,SEEK_SET);
+ str = malloc(l+2);
+ l=0;
+ while ((str[l++]=fgetc(f))!=EOF) ;
+ str[--l]=0;
+ gtk_text_insert(GTK_TEXT(txt), NULL,NULL,NULL,str,l);
+ fclose(f);
+ free(str);
+}
+
+static void save_proc(GtkWidget* w, gpointer txt)
+{
+ char *t,*filen;
+ char filename[MAXPATH];
+ FILE* f;
+ int l,i;
+ l = gtk_text_get_length(GTK_TEXT(txt));
+ if (l<3)
+    {
+     gtk_dialog_printf("Will not save empty query.");
+     return;
+    }
+ t = malloc(l+2);
+ for (i=0;i<l;i++) t[i] = GTK_TEXT_INDEX(GTK_TEXT(txt), i);
+ t[i] = 0;
+ filen = gtk_dialog_getfile("Enter filename to save query");
+ if (!strcmp(filen,""))
+   {
+    gtk_dialog_printf("Cannot save query to empty name.\n");
+    free(t);
+    return;
+   }
+ strcpy(filename,filen);
+ strcat(filename,".QUE");
+ f = fopen(filename,"r");
+ if (f)
+   {
+    fclose(f);
+    if (!gtk_dialog_yes_no_printf("Overwite file: %s?", filename)) 
+      {
+       free(t);
+       return ;
+      }
+   }
+ f = fopen(filename,"w"); 
+ if (!f)
+   {
+    gtk_dialog_printf("Cannot write to: %s\n", filename);
+    free(t);
+    return ;
+   }
+ fprintf(f,"%s", t);
+ free(t);
+ fclose(f);
+ gtk_dialog_printf("Saved to %s\n", filename);
+}
+
 static void execute_proc(GtkWidget* w, gpointer txt)
 {
  char* t;
@@ -249,11 +519,13 @@ static void execute_proc(GtkWidget* w, gpointer txt)
  free_p3c(&out, a, b);
 }
 
+
 void update_entry(GtkWidget* w, gpointer d)
 {
  strcpy((char*)d,gtk_entry_get_text(GTK_ENTRY(w)));
 }
 
+
 void apply_proc(GtkWidget* w, gpointer d)
 {
  char** ptr;
@@ -264,6 +536,7 @@ void apply_proc(GtkWidget* w, gpointer d)
  reload_bases(ptr[3], 1);
 }
 
+
 int connect_proc(GtkWidget* w, gpointer d)
 {
  char** ptr;
@@ -284,6 +557,7 @@ int delete_proc(GtkWidget* w, gpointer d)
  return TRUE;
 }
 
+
 char* get_env(char* str)
 {
  char* ret;
@@ -310,7 +584,7 @@ static void env_dialog(GtkWidget* www, gpointer d)
  label[1] = gtk_label_new("DataDir:");
  label[2] = gtk_label_new("Host:");
  label[3] = gtk_label_new("DataBase:");
- for (i=0;i<4;i++) 
+ for (i=0;i<4;i++)
   {
    entry[i] = gtk_entry_new();
    gtk_entry_set_max_length(GTK_ENTRY(entry[i]), STDSTRING-1);
@@ -318,13 +592,13 @@ static void env_dialog(GtkWidget* www, gpointer d)
    gtk_signal_connect(GTK_OBJECT(entry[i]), "activate", GTK_SIGNAL_FUNC(update_entry), (gpointer)t_entry[i]);
    gtk_signal_connect(GTK_OBJECT(entry[i]), "changed",  GTK_SIGNAL_FUNC(update_entry), (gpointer)t_entry[i]);
   }
- table = gtk_table_new (5, 3, TRUE); 
+ table = gtk_table_new (5, 3, TRUE);
  gtk_entry_set_text(GTK_ENTRY(entry[0]), get_env("PGPORT"));
  gtk_entry_set_text(GTK_ENTRY(entry[1]), get_env("PGDATA"));
  gtk_entry_set_text(GTK_ENTRY(entry[2]), get_env("PGHOST"));
  gtk_entry_set_text(GTK_ENTRY(entry[3]), currentbase?currentbase:"");
  for (i=0;i<4;i++)
-   { 
+   {
     gtk_table_attach (GTK_TABLE (table),label[i], 0, 1, i, i+1,GTK_FILL,GTK_FILL, 0, 0);
     gtk_table_attach (GTK_TABLE (table),entry[i], 1, 3, i, i+1,GTK_FILL|GTK_EXPAND,GTK_FILL, 0, 0);
     gtk_widget_show(entry[i]);
@@ -360,6 +634,7 @@ static void env_dialog(GtkWidget* www, gpointer d)
  gtk_widget_destroy(dialog);
 }
 
+
 static void check_exec_proc(GtkWidget* txt, gpointer dlg)
 {
  int l;
@@ -367,9 +642,10 @@ static void check_exec_proc(GtkWidget* txt, gpointer dlg)
  if (GTK_TEXT_INDEX(GTK_TEXT(txt), l-1)==';') execute_proc(dlg,(GtkWidget*)txt);
 }
 
+
 static void query_executor(GtkWidget* w, gpointer data)
 {
- GtkWidget* window, *table, *text, *hscrollbar, *vscrollbar, *button;
+ GtkWidget* window, *table, *text, *hscrollbar, *vscrollbar, *button[3];
  if (query_exec)
    {
     gtk_dialog_printf("Another instance of QueryExecutor is running.");
@@ -378,34 +654,41 @@ static void query_executor(GtkWidget* w, gpointer data)
  window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
  gtk_widget_set_usize (GTK_WIDGET(window), 400, 300);
  gtk_container_set_border_width(GTK_CONTAINER(window), 10);
- table = gtk_table_new (2, 2, FALSE);  
+ table = gtk_table_new (3, 4, FALSE);
  text = gtk_text_new (NULL, NULL);
- gtk_table_attach (GTK_TABLE (table), text, 0, 1, 0, 1,
+ gtk_table_attach (GTK_TABLE (table), text, 0, 3, 1, 2,
 		      GTK_FILL | GTK_EXPAND,
 		      GTK_FILL | GTK_EXPAND | GTK_SHRINK, 0, 0);
  gtk_text_set_editable(GTK_TEXT(text), TRUE);
  gtk_text_set_word_wrap(GTK_TEXT(text), TRUE);
  gtk_text_set_line_wrap(GTK_TEXT(text), TRUE);
- gtk_widget_show (text);   
+ gtk_widget_show (text);
  hscrollbar = gtk_hscrollbar_new (GTK_TEXT (text)->hadj);
- gtk_table_attach (GTK_TABLE (table), hscrollbar, 0, 1, 1, 2,
+ gtk_table_attach (GTK_TABLE (table), hscrollbar, 0, 3, 2, 3,
 		      GTK_EXPAND | GTK_FILL, GTK_FILL, 0, 0);
- gtk_widget_show (hscrollbar);  
+ gtk_widget_show (hscrollbar);
  vscrollbar = gtk_vscrollbar_new (GTK_TEXT (text)->vadj);
- gtk_table_attach (GTK_TABLE (table), vscrollbar, 1, 2, 0, 1,
+ gtk_table_attach (GTK_TABLE (table), vscrollbar, 3, 4, 1, 2,
 		      GTK_FILL, GTK_EXPAND | GTK_FILL | GTK_SHRINK, 0, 0);
  gtk_widget_show (vscrollbar);
- button = gtk_button_new_with_label("Go!");
- gtk_table_attach (GTK_TABLE (table), button, 1, 2, 1, 2,
-		      GTK_FILL, GTK_FILL, 0, 0);
- gtk_widget_show(button);
+ button[0] = gtk_button_new_with_label("Run");
+ button[1] = gtk_button_new_with_label("Save");
+ button[2] = gtk_button_new_with_label("Load");
+ gtk_table_attach (GTK_TABLE (table), button[0], 0, 1, 0, 1, GTK_FILL, GTK_FILL, 0, 0);
+ gtk_table_attach (GTK_TABLE (table), button[1], 1, 2, 0, 1, GTK_FILL, GTK_FILL, 0, 0);
+ gtk_table_attach (GTK_TABLE (table), button[2], 2, 3, 0, 1, GTK_FILL, GTK_FILL, 0, 0);
+ gtk_widget_show(button[0]);
+ gtk_widget_show(button[1]);
+ gtk_widget_show(button[2]);
  gtk_container_add (GTK_CONTAINER(window), table);
  gtk_widget_show (table);
  gtk_widget_grab_focus(text);
  gtk_signal_connect(GTK_OBJECT(window), "delete_event", GTK_SIGNAL_FUNC(qexec_quit), NULL);
  gtk_signal_connect(GTK_OBJECT(text),   "changed", GTK_SIGNAL_FUNC(check_exec_proc), (gpointer)(window));
  gtk_signal_connect(GTK_OBJECT(window), "destroy", GTK_SIGNAL_FUNC(qexec_quit), NULL);
- gtk_signal_connect(GTK_OBJECT(button), "clicked", GTK_SIGNAL_FUNC(execute_proc), (gpointer)(text));
+ gtk_signal_connect(GTK_OBJECT(button[0]), "clicked", GTK_SIGNAL_FUNC(execute_proc), (gpointer)(text));
+ gtk_signal_connect(GTK_OBJECT(button[1]), "clicked", GTK_SIGNAL_FUNC(save_proc), (gpointer)(text));
+ gtk_signal_connect(GTK_OBJECT(button[2]), "clicked", GTK_SIGNAL_FUNC(load_proc), (gpointer)(text));
  gtk_window_set_title(GTK_WINDOW(window), "SQL Query Editor, type `;' to execute query");
  gtk_widget_show(window);
  query_exec = window;
@@ -432,21 +715,23 @@ static void query_executor(GtkWidget* w, gpointer data)
            "<Separator>"      -> create a separator
            "<Branch>"         -> create an item to hold sub items (optional)
            "<LastBranch>"     -> create a right justified branch
-*/
+*/			/* FIXME */			/* ADD TOOLBAR WITH SHORTCUTS TO MENU */
+			/* FIXME */			/* TESTS, TOOLBARS, RELATIONS ... */
 static GtkItemFactoryEntry menu_items[] = {
   { "/_File",         		NULL,         NULL, 0, "<Branch>" },
-  { "/File/_New DB",     	"<control>N", create_database, 0, NULL },
-  { "/File/_Open DB",    	"<control>O", __dummy, 0, NULL },
-  { "/File/_Load DB from File", "<control>L", __dummy, 0, NULL },
-  { "/File/_Save DB to File",	"<control>S", save_database, 0, NULL },
-  { "/File/_Execute SQL Query",	"<control>E", query_executor, 0, NULL },
+  { "/File/_New DB",     	"<control>N", create_database, 0, NULL },	/* DONE */
+  { "/File/_Open DB",    	"<control>O", load_database, 0, NULL },		/* DONE */
+  { "/File/_Create DB from File","<control>C", create_DB_from_file, 0, NULL },  /* DONE */
+  { "/File/_Save DB to File",	"<control>S", save_database, 0, NULL },		/* DONE */
+  { "/File/_Drop DB",	"<control>D", drop_database, 0, NULL },			/* DONE */
+  { "/File/_Execute SQL Query",	"<control>E", query_executor, 0, NULL },	/* DONE */
   { "/File/sep1",     NULL,         NULL, 0, "<Separator>" },
-  { "/File/Quit",     "<control>Q", (void (*)(GtkWidget*,gpointer))menu_quit, 0, NULL },
+  { "/File/Quit",     "<control>Q", (void (*)(GtkWidget*,gpointer))menu_quit, 0, NULL }, /* DONE */
   { "/_Options",      NULL,         NULL, 0, "<Branch>" },
-  { "/Options/Test",  "<control>T",         exec_test_prog, 0, NULL },
-  { "/Options/Environment",     NULL,       env_dialog, 0, NULL },
+  { "/Options/Test",  "<control>T",         exec_test_prog, 0, NULL },		/* DONE */
+  { "/Options/Environment",     NULL,       env_dialog, 0, NULL },		/* DONE */
   { "/_Help",         NULL,         NULL, 0, "<LastBranch>" },
-  { "/_Help/About",   NULL,         about_box, 0, NULL },
+  { "/_Help/About",   NULL,         about_box, 0, NULL },			/* ADD PIXMAPS ETC. */
 };
 
 void get_main_menu( GtkWidget  *window,GtkWidget **menubar )
@@ -461,6 +746,22 @@ void get_main_menu( GtkWidget  *window,GtkWidget **menubar )
   if (menubar) *menubar = gtk_item_factory_get_widget (item_factory, "<main>");
   error("menu is not yet implemented.\n");
 }
+int dummy_connect(int cn, char* templ)
+{
+ char errstr[MAX_ENTRY_LENGTH];
+ if (cn==1)
+  {
+   connection = gtk_connect_db(templ, errstr);
+   if (!connection)
+     {
+      gtk_dialog_printf("Connect to template DB: %s failed\nDBMS returned:\n%s", templ,errstr);
+      return 0;
+     }
+   return 1;
+  }
+ disconnect();
+ return 0;
+}
 
 int parse_cmdline(int lb, char** par, GtkWidget* w)
 {
@@ -469,20 +770,35 @@ int parse_cmdline(int lb, char** par, GtkWidget* w)
  int gotdb,ch;
  gotdb=0;
  dbname = NULL;
- printf("Dodac opcje zapisac aktualnej kwerendy i wykonania kwerendy z pliku: opefiledialog\n");
- printf("Zmienic w savedb to file na uzycie savefiledialog, i tak samo uzyc w zapisie kwerendy\n");
- printf("powyzsze f-cje dodac do gtkdialogs.[ch]\n");
- while ((ch = getopt(lb,par,"d:he")) != -1)
+ while ((ch = getopt(lb,par,"d:s:p:b:he")) != -1)
    {
     switch (ch)
       {
+       case 's': setenv("PGHOST", optarg, 1); break;
+       case 'p': setenv("PGPORT", optarg, 1); break;
+       case 'b': setenv("PGDATA", optarg, 1); break;
        case 'd': gotdb=1; dbname = malloc(strlen(optarg)+1); strcpy(dbname, optarg); break;
        case 'e': gotdb=1; env_dialog(w,(gpointer)1); return 0;
-       case 'h': gtk_dialog_printf("Options are:\n-d [database_name]\n-e (use advanced environment)"); return 1;
+       case 'h':
+		 gtk_dialog_printf_big("Options are:\n-d [database_name]\n"
+				 "-e (use advanced environment)\n"
+				 "-h this help\n"
+				 "-s server name\n"
+				 "-p port\n"
+				 "-b base dir"
+				 ); return 1;
        default: gtk_dialog_printf("Unrecognized command line option\noptions are:\n-d data_base_name\n-h help"); return 1;
       }
    }
- if (!gotdb) dbname = gtk_dialog_gettext("Connect to database: ");
+ if (!gotdb) 
+    { 
+     if (dummy_connect(1,"template1")) 
+       {
+	dbname = gtk_dialog_getdbs(connection, "Connect to database: "); 
+        dummy_connect(0,"template1");
+       }
+     else dbname = gtk_dialog_gettext("Type DB name manually:");
+    }
  if (!strcmp(dbname,""))
     {
      gtk_dialog_printf("Empty Database name, cannot be opened.\nSystem will now exit.");
@@ -495,7 +811,7 @@ int parse_cmdline(int lb, char** par, GtkWidget* w)
     if (gotdb) free(dbname);
     return 1;
    }
- else 
+ else
    {
     currentbase = (char*)malloc(strlen(dbname)+2);
     strcpy(currentbase, dbname);
@@ -528,7 +844,7 @@ int main(int lb, char **par)
   main_wnd = window;
   err = parse_cmdline(lb,par,window);
   if (err) return 1;
-  else 
+  else
     {
      sprintf(winname, "PgXexplorer: %s", currentbase);
      gtk_window_set_title (GTK_WINDOW(window), winname);
