@@ -91,6 +91,56 @@ PGresult* execute_silent_query(PGconn* c,  char* query)
  return r;
 }
 
+char*** p3c_execute_printf(PGconn* c,int* row, int* col, char* errstr, char* fmt,...)
+{
+ va_list ap;
+ char*** ret;
+ char msg[MAX_QUERY_LENGTH];
+ va_start(ap, fmt);
+ vsprintf(msg,fmt,ap);
+ va_end(ap);
+ execute_printf(c, &ret, row, col, errstr, msg);
+ return ret;
+}
+
+void execute_printf(PGconn* c, char**** res, int* row, int* col, char* errstr, char* fmt,...)
+{
+ int stat;
+ va_list ap;
+ char query[MAX_QUERY_LENGTH];
+ PGresult* r;
+ va_start(ap, fmt);
+ vsprintf(query,fmt,ap);
+ va_end(ap);
+ r=NULL;
+ if (!query)
+   {
+    *res = NULL;
+    *row = *col = -1;
+    strcpy(errstr, "NULL query passed to DB");
+    return;
+   }
+ r= PQexec(c, query);
+ if (!r)
+   {
+    sprintf(errstr,"PQresult is NULL %s\nQuery was:\n%s\n", PQerrorMessage(c), query);
+    *res = NULL;
+    *row = *col = -1;
+    return ;
+   }
+ stat = PQresultStatus(r);
+ if (stat!=PGRES_COMMAND_OK && stat!=PGRES_TUPLES_OK)
+   {
+    sprintf(errstr,"PQexec:\n%s\n%s\nQuery was:\n%s",PQcmdStatus(r),PQresultErrorMessage(r), query);
+    *res = NULL;
+    *row = *col = -1;
+    clear_result(&r);
+    return;
+   }
+ *res = pgres2pc3(r, row, col);
+ clear_result(&r);
+}
+
 void execute_printf_query(PGconn* c,PGresult** r, char* errstr, char* fmt,...)
 {
  int stat;
@@ -115,6 +165,7 @@ void execute_printf_query(PGconn* c,PGresult** r, char* errstr, char* fmt,...)
  if (stat!=PGRES_COMMAND_OK && stat!=PGRES_TUPLES_OK)
    {
     sprintf(errstr,"PQexec:\n%s\n%s\nQuery was:\n%s",PQcmdStatus(*r),PQresultErrorMessage(*r), query);
+    clear_result(r);
     *r = NULL;
     return;
    }
